#include "network.h"

static std::list<LISTARG*> thislist;
static std::mutex Network_mtx, List_mtx; 

static void emit(int signo)
{
	List_mtx.lock();
	if(thislist.empty()) {
		List_mtx.unlock();
		return;
	}
	List_mtx.unlock();
	if(signo != EMITSIGNAL) {
		List_mtx.unlock();
		return ;
	}

	LISTARG	*arg;
	do {
		arg = thislist.front();
		arg->network->emit_signal(arg->response);

		List_mtx.lock();
		thislist.remove(arg);
		List_mtx.unlock();
		delete arg;
	}while(!thislist.empty());

}

NetWork::NetWork(std::string url, std::string mothod)
{
	if(!mothod.empty()) {
		m_mothod = mothod;
	} else {
		m_mothod = "GET";
	}
	if(!url.empty()) {
		set_url(url);
	}
	signal(EMITSIGNAL, emit);
}

NetWork::~NetWork()
{
}

bool NetWork::recv_server(int sockfd, pthread_t main_pid, Response *respon)
{
	char		buf[512];
	int		len, pos;
	struct 	timeval tv;
	std::string 	str, data;

	if(sockfd < -1) {
		std::cout << "not connect?" << std::endl;
		return false;
	}

	tv.tv_sec = 60;
	tv.tv_usec = 0;
	setsockopt(sockfd, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv));
	data.clear();
	while( (len = recv(sockfd, buf, sizeof(buf)- sizeof(char), 0)) > 0) {
		buf[len] = 0;
		data.append(buf, len);
	}
	if(errno == EWOULDBLOCK) {
		std::cout << "read server timeout" << std::endl;
		return false;
	}

	pos = respon->data_to_head(data);
	respon->data_to_text(data, pos);
	data.clear();

	LISTARG *arg = new LISTARG;
	arg->network = this;
	arg->response = respon;
	thislist.push_back(arg);
	pthread_kill(main_pid, EMITSIGNAL);
	
	return true;
}

Response* NetWork::get()
{
	create_requestHead();
	Response *response = new Response;
	std::thread th(&NetWork::start, this, pthread_self(), response);
	th.detach();

	return response;
}

int NetWork::Connect_to_server()
{
	int	err_code, sockfd;
	struct addrinfo hints, *res, *resave;

	if(m_host.empty()) {
		std::cout << "m_host is empty\n";
		return false;
	}
	bzero(&hints, sizeof(struct addrinfo));
	hints.ai_family = AF_INET;
	hints.ai_socktype = SOCK_STREAM;

	if( (err_code = getaddrinfo(m_host.c_str(), "80", &hints, &res)) != 0) {
		std::cout << "getaddrinfo error: " << gai_strerror(err_code) << std::endl;
		return false;
	}
	resave = res;

	do {
		if( (sockfd = socket(res->ai_family, res->ai_socktype, res->ai_protocol)) < 0) {
			continue;
		}

		if(connect(sockfd, res->ai_addr, res->ai_addrlen) == 0) {
			break;
		}

		close(sockfd);
	} while( (res = res->ai_next) != NULL);

	freeaddrinfo(resave);
	if(res == NULL) {
		std::cout << "connect error for " << m_host << std::endl;
		return false;
	}

	return sockfd;
}

bool NetWork::send_server(int sockfd)
{
	if(m_requestHead.empty()) {
		std::cout << "Request Head is emptr" << std::endl;
		return false;
	}
	m_requestHead += "\r\n";

	int size = m_requestHead.size();
	struct timeval tv;
	tv.tv_sec = 60;
	tv.tv_usec = 0;
	setsockopt(sockfd, SOL_SOCKET, SO_SNDTIMEO, &tv, sizeof(tv));
	if(send(sockfd, m_requestHead.c_str(), size, 0) < size) {
		if(errno == EWOULDBLOCK) {
			std::cout << "Write server timeout" << std::endl;
		} else {
			std::cout << "Write the data does not meet the requirements" << std::endl;
		}
		return false;
	}
	return true;
}
void NetWork::create_requestHead()
{
	m_requestHead =  Glib::ustring::compose("%1 %2 HTTP/1.1\r\n", m_mothod, m_path);
	m_requestHead +=  Glib::ustring::compose("Host: %1\r\n", m_host);
	m_requestHead +=  "User-Agent: Hello World(Linux)\r\n";
	if(m_append.empty()) {
		return;
	}
	m_requestHead += m_append;
}

bool NetWork::set_url(const std::string &url)
{
	if(url.empty()) {
		return false;
	}
	if(!analysis_url(url)) {
		return false;
	}
	return true;
}

void NetWork::emit_signal(Response *response)
{
	m_signal_something.emit(response);
}

void NetWork::append_request_head(const std::string &headerName, const std::string &headerValue)
{
/*	if(m_requestHead.empty()) {
		std::cout << "m_requesrHead.empty() return true" << std::endl;
		return;
	}*/
	m_append += Glib::ustring::compose("%1: %2\r\n", headerName, headerValue);
}

NetWork::type_signal_something NetWork::signal_something()
{
	return m_signal_something;
}

bool NetWork::analysis_url(const std::string &url)
{
	int pos = 0;

	if( (pos = url.find("http://", 0)) == std::string::npos) {
		return false;
	}
	pos += Strcpy(&m_host, url, pos += 7, '/');

	Strcpy(&m_path, url, pos, '\0');

	return true;
}

void NetWork::start(pthread_t pid, Response *respon)
{
	//锁
	Network_mtx.lock();
	int sockfd, i;
	if( (sockfd = Connect_to_server()) <= 0) {
		return;
	}
	for(i = 0 ; i < 3; ++i) {
		if(send_server(sockfd)) {
			break;
		}
	}
	Network_mtx.unlock();
	if(i == 3) {
		std::cout << "The last attempt a failure" << std::endl;
		return;
	}

	recv_server(sockfd, pid, respon);
	//解锁
}
/*
void NetWork::recv_server(int sockfd)
{	
	NetWork *http = ((THISARG*)arg)->network;
	int sockfd, i = 0;



	pthread_t pid;
	pid = ((THISARG*)arg)->thread_main_id;
	delete ((THISARG*)arg);

	read_http(sockfd, pid, http);
	close(sockfd);
	delete ((THISARG*)arg);
	pthread_mutex_unlock(&thisattr);
	pthread_exit((void*)true);
}
*/

void NetWork::set_mothod(const std::string &mothod)
{
	m_mothod = mothod;
}
